@@ -13,7 +13,17 @@ def list_models():
   return {
     "circles": "/circles/",
     "circlesjoinable": "/circles/joinable/",
+    "communities": "/communities/",
+    "opencommunities": "/open-communities/",
+    "communitiesaddresses": "/communities-addresses/",
     "dashboards": "/dashboards/",
+    "events": "/events/",
+    "eventsfuture": "/events/future/",
+    "eventspast": "/events/past/",
+    "typeevents": "/typeevents/",
+    "resources": "/resources/",
+    "keywords": "/keywords/",
+    "types": "/types/",
     "joboffers": "/job-offers/current/",
     "projects": "/projects/",
     "projectsjoinable": "/projects/joinable/",
